from pathlib import Path
from ursus.config import config
from ursus.context_processors import Context, EntryContextProcessor, EntryURI
from ursus.utils import (
    is_raster_image,
    get_image_size,
    get_image_transforms,
    is_image,
    is_pdf,
)


class ImageProcessor(EntryContextProcessor):
    def process_entry(
        self,
        context: Context,
        entry_uri: EntryURI,
        changed_files: set[Path] | None = None,
    ) -> None:
        if (
            config.fast_rebuilds
            and changed_files
            and (config.content_path / entry_uri) not in changed_files
        ):
            return

        abs_path = config.content_path / entry_uri
        if is_raster_image(abs_path):
            width, height = get_image_size(abs_path)
            context["entries"][entry_uri].update(
                {
                    "width": width,
                    "height": height,
                }
            )
        if is_image(abs_path) or is_pdf(abs_path):
            context["entries"][entry_uri].update(
                {
                    "transforms": list(get_image_transforms(Path(entry_uri))),
                }
            )



from pathlib import Path
from ursus.config import config
from ursus.linters import RegexLinter, LinterResult
import logging
import re


class OrphanFootnotesLinter(RegexLinter):
    """
    Count footnotes; make sure that they all appear at least twice
    """

    file_suffixes = (".md",)
    regex = re.compile(r"(?P<footnote>\[\^(?P<id>\d+)\])(?P<colon>:?)")

    def lint(self, file_path: Path) -> LinterResult:
        if self.file_suffixes and file_path.suffix.lower() not in self.file_suffixes:
            return

        footnotes: dict[str, list[dict]] = {}

        with (config.content_path / file_path).open() as file:
            for line_no, line in enumerate(file.readlines()):
                for match in self.regex.finditer(line):
                    footnotes.setdefault(match["id"], [])
                    footnotes[match["id"]].append(
                        {
                            "position": (line_no, *match.span()),
                            "is_definition": bool(match["colon"]),
                        }
                    )

        for footnote_id, occurences in footnotes.items():
            definitions = [o for o in occurences if o["is_definition"]]

            if len(definitions) == 0:
                for occurence in occurences:
                    yield (
                        occurence["position"],
                        f"Undefined footnote: [^{footnote_id}]",
                        logging.ERROR,
                    )
            elif len(definitions) > 1:
                for definition in definitions:
                    yield (
                        definition["position"],
                        f"Multiple footnote definitions: [^{footnote_id}]",
                        logging.ERROR,
                    )
            elif len(definitions) == len(occurences):
                for occurence in occurences:
                    yield (
                        occurence["position"],
                        f"Unused footnote: [^{footnote_id}]",
                        logging.ERROR,
                    )



from ursus.context_processors.get_entries import get_entries


entries = {
    "blog/hello.md": {
        "title": "Hello world",
        "month": "January",
        "short_title": "Hello",
        "description": "First post on this blog!",
    },
    "blog/bonjour.md": {
        "title": "Bonjour monde",
        "month": "October",
        "short_title": "Bonjour",
        "description": "Deuxième post sur ce blogue",
    },
    "blog/hallo.md": {
        "title": "Hallo Welt",
        "month": "September",
        "short_title": "Hallo",
        "description": "Dritter Beitrag auf diesem Blog",
    },
    "recipes/pancakes.md": {
        "title": "Pancake recipe",
        "description": "Flat and delicious",
        "ingredients": ["Flour", "eggs", "milk"],
    },
    "home.md": {
        "title": "Home page",
        "content": "Boy, what a home page!",
    },
    "empty.md": {},
}


def test_sort_by_function():
    def month_getter(entry):
        return [
            "January",
            "February",
            "March",
            "April",
            "May",
            "June",
            "July",
            "August",
            "September",
            "October",
            "November",
            "December",
        ].index(entry["month"])

    assert get_entries(entries, "blog", sort_by=month_getter) == [
        entries["blog/hello.md"],
        entries["blog/hallo.md"],
        entries["blog/bonjour.md"],
    ]


def test_sort_by_one_key():
    assert get_entries(entries, "blog", sort_by="title") == [
        entries["blog/bonjour.md"],
        entries["blog/hallo.md"],
        entries["blog/hello.md"],
    ]


def test_sort_by_multiple_keys():
    assert get_entries(entries, "blog", sort_by=["short_title", "title"]) == [
        entries["blog/bonjour.md"],
        entries["blog/hallo.md"],
        entries["blog/hello.md"],
    ]


def test_filter_by():
    def uri_length_filter(entry_uri, entry):
        return len(entry_uri) <= 13

    filtered_entries = get_entries(entries, None, filter_by=uri_length_filter)
    assert len(filtered_entries) == 4
    assert entries["blog/hallo.md"] in filtered_entries
    assert entries["blog/hello.md"] in filtered_entries
    assert entries["home.md"] in filtered_entries
    assert entries["empty.md"] in filtered_entries


def test_no_namespace():
    assert len(get_entries(entries)) == 6


def test_namespace():
    assert get_entries(entries, "recipes") == [
        entries["recipes/pancakes.md"],
    ]



from setuptools import setup, find_packages
from pathlib import Path

root_dir = Path(__file__).parent

long_description = (root_dir / "README.md").read_text()
requirements_path = root_dir / "requirements.txt"
install_requires = requirements_path.read_text().strip().split("\n")

setup(
    name="ursus_ssg",
    version="1.4.5",
    description="Static site generator",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/all-about-berlin/ursus",
    author="Nicolas Bouliane",
    author_email="contact@nicolasbouliane.com",
    license="MIT",
    packages=find_packages(),
    scripts=["bin/ursus"],
    entry_points={
        "markdown.extensions": [
            "better_footnotes = ursus.context_processors.markdown:FootnotesExtension",
            "jinja = ursus.context_processors.markdown:JinjaExtension",
            "base_url = ursus.context_processors.markdown:BaseUrlExtension",
            "responsive_images = ursus.context_processors.markdown:ResponsiveImagesExtension",
            "superscript = ursus.context_processors.markdown:SuperscriptExtension",
            "tasklist = ursus.context_processors.markdown:TaskListExtension",
        ]
    },
    package_data={
        "ursus": [
            "py.typed",
        ],
    },
    python_requires=">=3.11",
    install_requires=install_requires,
    zip_safe=False,
)
